@@ -56,6 +56,10 @@ YAJL_ERROR = 3
 class JSONError(Exception):
     pass
 
+class IncompleteJSONError(JSONError):
+    def __init__(self):
+        super(IncompleteJSONError, self).__init__('Incomplete or empty JSON data')
+
 def basic_parse(f, allow_comments=False, check_utf8=False, buf_size=64 * 1024):
     '''
     An iterator returning events from a JSON being parsed. This basic parser
@@ -90,7 +94,7 @@ def basic_parse(f, allow_comments=False, check_utf8=False, buf_size=64 * 1024):
             return 1
         return func_type(c_callback)
 
-    yajl.yajl_get_error.restype = c_char_p
+    yajl.yajl_get_error.restype = c_void_p
     callbacks = Callbacks(*[callback(*data) for data in _callback_data])
     config = Config(allow_comments, check_utf8)
     handle = yajl.yajl_alloc(byref(callbacks), byref(config), None, None)
@@ -101,15 +105,19 @@ def basic_parse(f, allow_comments=False, check_utf8=False, buf_size=64 * 1024):
                 result = yajl.yajl_parse(handle, buffer, len(buffer))
             else:
                 result = yajl.yajl_parse_complete(handle)
-            if not buffer or (result in (YAJL_CANCELLED, YAJL_ERROR)):
+            if result == YAJL_ERROR:
+                perror = yajl.yajl_get_error(handle, 1, buffer, len(buffer))
+                error = c_char_p(perror).value
+                yajl.yajl_free_error(handle, perror)
+                raise JSONError(error)
+            if not buffer and not events:
+                if result == YAJL_INSUFFICIENT_DATA:
+                    raise IncompleteJSONError()
                 break
-            for event, value in events:
-                yield event, value
-            events = []
 
-        if result == YAJL_ERROR:
-            error = yajl.yajl_get_error(handle, 1, buffer, len(buffer))
-            raise JSONError(error)
+            for event in events:
+                yield event
+            events = []
     finally:
         yajl.yajl_free(handle)
 
